NAME = "roofai"

ICON = "🏛️"

DESCRIPTION = f"{ICON} everything ai about roofs."

VERSION = "6.151.1"

MARQUEE = "https://github.com/kamangir/assets/blob/main/2023-10-28-16-28-36-88493-predict.gif?raw=true"

REPO_NAME = NAME

ALIAS = NAME


def fullname() -> str:
    return f"{NAME}-{VERSION}"



import os

from blue_objects import objects, file


def cache_pretrainedmodels() -> bool:
    HOME = os.getenv("HOME")

    filename = f"{HOME}/.cache/torch/hub/checkpoints/se_resnext50_32x4d-a260b3a4.pth"

    model_name = file.name(filename)

    if not objects.download(model_name):
        return False

    return file.copy(
        objects.path_of(
            filename=f"{model_name}.pth",
            object_name=model_name,
        ),
        filename,
        overwrite=False,
    )



from typing import List

from blue_options.terminal import show_usage, xtra

from roofai.roboflow.create import list_of_project_types


project_type_details = {
    "type: {}".format(" | ".join(list_of_project_types)): [],
}


def help_create_project(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("dryrun,", mono=mono),
            "project=<project-name>",
        ]
    )

    args = [
        "[--description <description>]",
        "[--type <type>]",
        "[--license <MIT>]",
    ]

    return show_usage(
        [
            "@roboflow",
            "create_project",
            f"[{options}]",
        ]
        + args,
        "create a roboflow project.",
        project_type_details,
        mono=mono,
    )


def upload_options(
    mono: bool,
    cascade: bool = False,
):
    return "".join(
        [
            xtra("~create,", mono=mono),
            "" if cascade else xtra("~download,", mono=mono),
            xtra("dryrun,", mono=mono),
            "project=<project-name>",
        ]
    )


def help_status(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "dryrun"

    return show_usage(
        [
            "@roboflow",
            "status",
            f"[{options}]",
        ],
        "show roboflow status.",
        mono=mono,
    )


def help_upload(
    tokens: List[str],
    mono: bool,
) -> str:
    options = upload_options(mono=mono)

    return show_usage(
        [
            "@roboflow",
            "upload",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "<object-name> -> roboflow/<project-name>.",
        mono=mono,
    )


help_functions = {
    "create_project": help_create_project,
    "status": help_status,
    "upload": help_upload,
}



import os

from blue_objects import file, README

from roofai import NAME, VERSION, ICON, REPO_NAME
from roofai.dataset.README import items as dataset_items


items = README.Items(
    [
        {
            "name": "Datasets",
            "url": "./roofai/dataset",
            "marquee": "https://github.com/kamangir/assets/blob/main/roofAI/AIRS-cache-v45--review-index-2.png?raw=true",
            "description": "Semantic Segmentation Datasets",
        },
        {
            "name": "Semantic Segmentation (SemSeg)",
            "url": "./roofai/semseg",
            "marquee": "./assets/predict-00247.png",
            "description": "A Semantic Segmenter based on [segmentation_models.pytorch](<https://github.com/qubvel/segmentation_models.pytorch/blob/master/examples/cars%20segmentation%20(camvid).ipynb>).",
        },
        {
            "name": "Google Maps API",
            "url": "./roofai/google_maps/api",
            "marquee": "https://github.com/kamangir/assets/blob/main/static-image-api-2025-02-15-wnfsd9/static-image-api-2025-02-15-wnfsd9-2X.gif?raw=true",
            "description": "Integrations with the Google Maps [Static](https://developers.google.com/maps/documentation/maps-static/start) and [Geocoding](https://developers.google.com/maps/documentation/geocoding/start) APIs.",
        },
        {
            "name": "SemSeg on Google Maps",
            "url": "./roofai/google_maps/semseg",
            "marquee": "https://github.com/kamangir/assets/raw/main/roofAI/roboflow/labelling-2.png?raw=true",
            "description": "Google Maps semantic segmentation datasets and models.",
        },
    ]
)


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            cols=readme.get("cols", 3),
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
        )
        for readme in [
            {
                "items": items,
                "cols": 2,
                "path": "..",
            },
            {
                "items": dataset_items,
                "cols": len(dataset_items),
                "path": "dataset",
            },
            {
                "path": "semseg",
            },
            {
                "path": "google_maps/api",
            },
            {
                "path": "google_maps/semseg",
            },
        ]
    )



import argparse
from tqdm import trange

from blueness import module
from blueness.argparse.generic import sys_exit

from roofai import NAME
from roofai.dataset.classes import RoofAIDataset
from roofai.dataset.classes import DatasetTarget
from roofai.dataset.ingest.from_dataset import ingest_from_dataset
from roofai.dataset.ingest.CamVid import ingest_CamVid
from roofai.logger import logger


NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="ingest|review",
)
parser.add_argument(
    "--input_dataset_path",
    type=str,
    default="",
)
parser.add_argument(
    "--count",
    type=int,
    default=1,
)
parser.add_argument(
    "--dataset_path",
    type=str,
    default="",
)
parser.add_argument(
    "--description",
    type=str,
    help="line1,line2",
    default="",
)
parser.add_argument(
    "--index",
    type=int,
    default=10,
)
parser.add_argument(
    "--output_dataset_path",
    type=str,
    default="",
)
parser.add_argument(
    "--log",
    type=int,
    default=1,
    help="0|1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--source",
    type=str,
    default="",
    help="AIRS|CamVid",
)
parser.add_argument(
    "--target",
    type=str,
    default="torch",
    help="torch|sagemaker",
)
parser.add_argument(
    "--subset",
    type=str,
    default="test",
)
parser.add_argument(
    "--test_count",
    type=int,
    default=10,
)
parser.add_argument(
    "--train_count",
    type=int,
    default=10,
)
parser.add_argument(
    "--val_count",
    type=int,
    default=10,
)
parser.add_argument(
    "--is_distributed",
    type=int,
    default=0,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "ingest":
    if args.source == "AIRS" or args.is_distributed == 1:
        success = ingest_from_dataset(
            input_dataset_path=args.input_dataset_path,
            output_dataset_path=args.output_dataset_path,
            counts={
                "test": args.test_count,
                "train": args.train_count,
                "val": args.val_count,
            },
            target=DatasetTarget[args.target.upper()],
            log=args.log,
            is_distributed=args.is_distributed,
            verbose=args.verbose,
        )
    elif args.source == "CamVid":
        success = ingest_CamVid(
            output_dataset_path=args.output_dataset_path,
        )
    else:
        logger.error(f"-{NAME}: {args.task}: {args.source}: source not found.")
elif args.task == "review":
    dataset = RoofAIDataset(args.dataset_path)

    for index in trange(args.count):
        dataset.visualize(
            subset=args.subset,
            index=index + args.index,
            in_notebook=False,
            description=args.description.split(","),
            log=args.log,
        )

    success = True
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import math
from tqdm import trange

from blueness import module
from blue_objects import objects
from blue_objects.metadata import post_to_object

from roofai import NAME
from roofai.google_maps.api.static import get as get_static_image
from roofai.google_maps.semseg.utils import meters_to_degrees
from roofai.logger import logger

NAME = module.name(__file__, NAME)


def ingest_dataset(
    lat: float,
    lon: float,
    zoom: int,
    maptype: str,
    size: str,
    count: int,
    object_name: str,
) -> bool:
    if count < 1:
        return False

    metadata = {
        "lat": lat,
        "lon": lon,
        "zoom": zoom,
        "maptype": maptype,
        "size": size,
        "count": count,
    }

    logger.info(
        "{}.ingest_dataset: [lat={:.6f}, lon={:.6f}] of {} @ zoom={} * count={} * {} -> {}".format(
            NAME,
            lat,
            lon,
            maptype,
            zoom,
            count,
            size,
            object_name,
        )
    )

    success, _, metadata["center"] = get_static_image(
        lat=lat,
        lon=lon,
        zoom=zoom,
        maptype=maptype,
        size=size,
    )
    if not success:
        return False

    width_count = int(math.ceil(math.sqrt(count)))
    height_count = int(math.ceil(count / width_count))
    logger.info(f"grid: {height_count}x{width_count}")
    metadata["grid"] = [height_count, width_count]

    chip_height_in_degrees, chip_width_in_degrees = metadata["center"]["size"]["deg"]

    lat_0 = lat - width_count / 2 * chip_width_in_degrees
    lon_0 = lon - height_count / 2 * chip_height_in_degrees

    for x in trange(width_count):
        for y in range(height_count):
            filename = objects.path_of(
                object_name=object_name,
                filename=f"{x:05d}-{y:05d}.png",
            )

            lat_xy = lat_0 + x * chip_width_in_degrees
            lon_xy = lon_0 + y * chip_height_in_degrees

            success, _, _ = get_static_image(
                lat=lat_xy,
                lon=lon_xy,
                filename=filename,
                zoom=zoom,
                maptype=maptype,
                size=size,
            )
            if not success:
                return False

    return post_to_object(
        object_name,
        NAME.replace(".", "-"),
        metadata,
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from roofai import NAME
from roofai.roboflow.create import create_project
from roofai.roboflow.status import get_status
from roofai.roboflow.upload import upload_to_project
from roofai.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="create_project | get_status | upload",
)
parser.add_argument(
    "--project_name",
    type=str,
)
parser.add_argument(
    "--description",
    type=str,
)
parser.add_argument(
    "--type",
    type=str,
    default="semantic-segmentation",
)
parser.add_argument(
    "--license",
    type=str,
    default="MIT",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0 | 1",
)
parser.add_argument(
    "--create",
    type=int,
    default=1,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "create_project":
    success = create_project(
        project_name=args.project_name,
        project_description=args.description,
        project_type=args.type,
        project_license=args.license,
    )
elif args.task == "get_status":
    success, _ = get_status()
elif args.task == "upload":
    success = upload_to_project(
        object_name=args.object_name,
        project_name=args.project_name,
        create=args.create == 1,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os

from blue_objects import file

from roofai import NAME, VERSION
from roofai.logger import logger

CLASSES = [
    "sky",
    "building",
    "pole",
    "road",
    "pavement",
    "tree",
    "signsymbol",
    "fence",
    "car",
    "pedestrian",
    "bicyclist",
    "unlabelled",
]


def ingest_CamVid(output_dataset_path: str) -> bool:
    logger.info(f"ingesting CamVid -> {output_dataset_path}")

    return file.save_yaml(
        os.path.join(output_dataset_path, "metadata.yaml"),
        {
            "classes": CLASSES,
            "kind": "CamVid",
            "source": "CamVid",
            "ingested-by": f"{NAME}-{VERSION}",
        },
        log=True,
    )
